(* Handling commandline options *)

open Global

(* Argument processing *)
let usage_string = "usage: " ^ Sys.argv.(0) ^ " options"

(* Set debug flag in Global module *)
let set_debug () =
  debug_flag := true

(* Set debug flag in Global module *)
let set_wait w =
  Global.wait := w

(* Set stdio flag *)
let stdio_flag = ref false
let set_stdio () =
  stdio_flag := true

(* Set minor heap to x kilo words *)
let set_minor_heap_size x =
  let gcControl = Gc.get () in
  gcControl.Gc.minor_heap_size <- x * 1024;
  Gc.set gcControl

let speclist = [
  "-ms", Arg.Int set_minor_heap_size, "Set minor heap size to x kilo words (default: x = 32)";
  "-d", Arg.Unit set_debug, "Set debug flag (default: off)";
  "-c", Arg.Unit set_stdio, "Use stdin/out instead of TCP/IP (default: off)";
  "-w", Arg.Float set_wait, "wait in seconds (default: 0.01)"
]

(* Entry point of the whole program *)
let _ =
  let alist = ref [] in
  Arg.parse speclist (fun x -> alist := x::!alist) usage_string;
  (* write down what you want *)
  Main.main !stdio_flag (Array.of_list (List.rev (!alist)))
