%{
open Types
%}

%start start0

%token ID EOL LETTER_N LETTER_S LETTER_E LETTER_W LETTER_P LETTER_D LETTER_X LETTER_Y EOF
%token <int> INT ID
%token <string> ROW

%type <unit> start0

%%

start0:
  start { () }
;
start:
  initial { $1 }
| start turn { $2 $1 }
;
initial:
  dimension rows config {
    let mapstring =
      let edge = String.make (fst($1) + 3) '#' in
      edge ^ (String.concat "##" $2) ^ edge in
    let map = new Smap.map $1 mapstring in
    let packagedb = new Robot.packagedb $1 in
    let robotdb = new Robot.robotdb $1 packagedb in
    let myrobot = new Robot.myrobot $3 map robotdb packagedb in
    robotdb#add (myrobot :> Robot.robot);
    (map, packagedb, robotdb, myrobot)
  }
;
dimension:
  INT INT EOL { ($1, $2) }
;
rows:
  { [] }
| ROW EOL rows { $1::$3 }
;
config:
  INT INT INT EOL { ($1, $2, $3) }
;

turn:
  EOF { failwith "Robot killed" }
| response package {
    fun (map, packagedb, robotdb, myrobot) ->
      robotdb#update $1;
      World.debug map packagedb robotdb;
      Global.send (myrobot#think $2);
      Unix.select [] [] [] (!Global.wait);
      (map, packagedb, robotdb, myrobot)
  }
;

response:
  EOL { [] }
| ID actions response { ($1, $2)::$3 }
;
actions:
  { [] }
| action actions { $1::$2 }
;
action:
  LETTER_N { N }
| LETTER_S { S }
| LETTER_E { E }
| LETTER_W { W }
| LETTER_P INT { PID $2 }
| LETTER_D INT { DID $2 }
| LETTER_X INT LETTER_Y INT { XY ($2, $4) }
;
package:
  EOL { [] }
| INT INT INT INT package { {package_id = $1; dest_x = $2; dest_y = $3; weight = $4}::$5 }
;

%%
